@@ -21,17 +21,11 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#include "test.h"
+#include "first.h"
 
-#include <fcntl.h>
-
-#include "testutil.h"
-#include "warnless.h"
 #include "memdebug.h"
 
-#define TEST_HANG_TIMEOUT 60 * 1000
-
-CURLcode test(char *URL)
+static CURLcode test_lib525(const char *URL)
 {
   CURLcode res = CURLE_OK;
   CURL *curl = NULL;
@@ -44,20 +38,14 @@ CURLcode test(char *URL)
   start_test_timing();
 
   if(!libtest_arg2) {
-#ifdef LIB529
-    /* test 529 */
-    curl_mfprintf(stderr, "Usage: lib529 [url] [uploadfile]\n");
-#else
-    /* test 525 */
-    curl_mfprintf(stderr, "Usage: lib525 [url] [uploadfile]\n");
-#endif
+    curl_mfprintf(stderr, "Usage: test [url] [uploadfile]\n");
     return TEST_ERR_USAGE;
   }
 
-  hd_src = fopen(libtest_arg2, "rb");
+  hd_src = curlx_fopen(libtest_arg2, "rb");
   if(!hd_src) {
     curl_mfprintf(stderr, "fopen failed with error (%d) %s\n",
-            errno, strerror(errno));
+                  errno, strerror(errno));
     curl_mfprintf(stderr, "Error opening file '%s'\n", libtest_arg2);
     return TEST_ERR_FOPEN;
   }
@@ -71,15 +59,15 @@ CURLcode test(char *URL)
   if(hd == -1) {
     /* can't open file, bail out */
     curl_mfprintf(stderr, "fstat() failed with error (%d) %s\n",
-            errno, strerror(errno));
+                  errno, strerror(errno));
     curl_mfprintf(stderr, "Error opening file '%s'\n", libtest_arg2);
-    fclose(hd_src);
+    curlx_fclose(hd_src);
     return TEST_ERR_FSTAT;
   }
 
   res_global_init(CURL_GLOBAL_ALL);
   if(res) {
-    fclose(hd_src);
+    curlx_fclose(hd_src);
     return res;
   }
 
@@ -145,24 +133,23 @@ CURLcode test(char *URL)
 
 test_cleanup:
 
-#ifdef LIB529
-  /* test 529 */
-  /* proper cleanup sequence - type PA */
-  curl_multi_remove_handle(m, curl);
-  curl_multi_cleanup(m);
-  curl_easy_cleanup(curl);
-  curl_global_cleanup();
-#else
-  /* test 525 */
-  /* proper cleanup sequence - type PB */
-  curl_multi_remove_handle(m, curl);
-  curl_easy_cleanup(curl);
-  curl_multi_cleanup(m);
-  curl_global_cleanup();
-#endif
+  if(testnum == 529) {
+    /* proper cleanup sequence - type PA */
+    curl_multi_remove_handle(m, curl);
+    curl_multi_cleanup(m);
+    curl_easy_cleanup(curl);
+    curl_global_cleanup();
+  }
+  else { /* testnum == 525 */
+    /* proper cleanup sequence - type PB */
+    curl_multi_remove_handle(m, curl);
+    curl_easy_cleanup(curl);
+    curl_multi_cleanup(m);
+    curl_global_cleanup();
+  }
 
   /* close the local file */
-  fclose(hd_src);
+  curlx_fclose(hd_src);
 
   return res;
 }
